"""
    MoinMoin - IncludeUrl macro

    Copyright (c) 2000-2001 by Richard Jones <richard@bizarsoftware.com.au>
    Copyright (c) 2000-2001 by J?rgen Hermann <jh@web.de>
    Copyright (c) 2001 by Conectiva, Inc.
    All rights reserved, see COPYING for details.

    This macro includes the formatted content of the given url, following
    recursive includes if encountered. Cycles are detected! It has been
    implemented by Gustavo Niemeyer <niemeyer@conectiva.com> by modifying
    the code of Include macro.

    Usage:
        [[IncludeUrl(url,heading,level)]]

        url         Name of the page to include
        heading     Text for the generated heading (optional)
        level       Level (1..5) of the generated heading (optional)
"""

import sys, string, re, cStringIO
from MoinMoin import user, config
from MoinMoin.Page import Page

class PageUrl(Page):
    def split_title(self):
        return self.page_name

    def exists(self):
        return 1

    def link_to(self, text=None):
        """Return HTML markup that links to this page"""
        word = self.page_name
        text = text or word
        return '<a href="%s">%s</a>'%(self.page_name, text)

    def get_raw_body(self):
        """Load the raw markup from the page file"""
        if self.raw_body is not None:
            return self.raw_body

        import urllib
        try:
            file = urllib.URLopener().open(self.page_name)
        except IOError, er:
            return ""

        # read file content and make sure it is closed properly
        try:
            result = file.read()
        finally:
            file.close()

        return result

_arg_heading = r'(?P<heading>,)\s*(|(?P<hquote>[\'"])(?P<htext>.+?)(?P=hquote))'
_arg_level = r',\s*(?P<level>\d+)'
_args_re_pattern = r'^(?P<name>[^,]+)(%s(%s)?)?$' % (_arg_heading, _arg_level)

def execute(macro, text, args_re=re.compile(_args_re_pattern)):
    ret = ''

    # parse and check arguments
    args = args_re.match(text)
    if not args:
        return ('<p><strong class="error">%s</strong></p>' % user.current.text('Invalid include arguments "%s"!')) % (text,)

    # get the page
    print_mode = macro.form.has_key('action') and macro.form['action'].value == "print"
    inc_name = args.group('name')
    this_page = macro.formatter.page
    if not hasattr(this_page, '_macroInclude_pagelist'):
        this_page._macroInclude_pagelist = {}
    if this_page._macroInclude_pagelist.has_key(inc_name):
        ret = ret + '<p><strong class="error">Recursive include of "%s" forbidden</strong></p>' % (inc_name,)
    inc_page = PageUrl(inc_name, formatter=macro.formatter.__class__())
    inc_page._macroInclude_pagelist = this_page._macroInclude_pagelist

    # do headings
    level = None
    if args.group('heading'):
        heading = args.group('htext') or inc_page.split_title()
        level = 1
        if args.group('level'):
            level = int(args.group('level'))
        if print_mode:
            ret = ret + macro.formatter.heading(level, heading)
        else:
            ret = ret + inc_page.link_to(macro.formatter.heading(level, heading))

    # set or increment include marker
    this_page._macroInclude_pagelist[inc_name] = \
        this_page._macroInclude_pagelist.get(inc_name, 0) + 1

    # output the included page
    stdout = sys.stdout
    sys.stdout = cStringIO.StringIO()
    inc_page.send_page(macro.form, content_only=1)
    ret = ret + sys.stdout.getvalue()
    sys.stdout = stdout

    # decrement or remove include marker
    if this_page._macroInclude_pagelist[inc_name] > 1:
        this_page._macroInclude_pagelist[inc_name] = \
            this_page._macroInclude_pagelist[inc_name] - 1
    else:
        del this_page._macroInclude_pagelist[inc_name]

    # if no heading and not in print mode, then output a helper link
    if not (level or print_mode):
        ret = ret + inc_page.link_to(user.current.text('<small>[goto %s]</small>') % (inc_name,))

    # return include text
    return ret
