"""
    MoinMoin - DeleteUploadedFiles action

    Copyright (c) 2001 by Jrgen Hermann <jh@web.de>
    All rights reserved, see COPYING for details.

    This action allows you to delete a page. Note that the standard
    config lists this action as excluded!

    $Id: DeletePage.py,v 1.3 2001/03/30 21:06:52 jhermann Exp $
"""

# Imports
import string, os, shutil
from MoinMoin import config, user, webapi, wikiutil
from MoinMoin.Page import Page


def execute(pagename, form):
    actname = string.split(__name__, '.')[-1]
    page = Page(pagename)

    # be extra paranoid in dangerous actions
    if actname in config.excluded_actions:
        return page.send_page(form,
            msg='<strong>%s</strong>' %
                user.current.text('You are not allowed to delete uploaded files!'))
            

    # check if the user is logged on
    if config.edit_requires_login and (not user.current.valid):
        return page.send_page(form,
            msg='<strong>%s</strong>' %
                user.current.text('<b>Please <a href="UserPreferences">log in</a> before deleting</b>')) #%webapi.getScriptname()))

    # check whether the user clicked the delete button
    if form.has_key('button'):
        # check whether this is a valid deletion request (make outside
        # attacks harder by requiring two full HTTP transactions)
        if not form.has_key('pagename'):
            return page.send_page(form,
                msg='<strong>%s</strong>' %
                    user.current.text('Please use the interactive user interface from UploadedFiles macro to delete files!'))
        if not form.has_key('files'):
            return page.send_page(form,
                msg='<strong>%s</strong>'%
                    user.current.text('Please check the file(s) to delete!'))
        # Delete the files
        deletedFilesNo=deleteFiles(form['files'])
        if not deletedFilesNo:
            return Page(pagename).send_page(form,
                    msg='<strong>%s</strong>' %
                        (user.current.text('No file was deleted!')))
        # Redirect to the current page
        return Page(pagename).send_page(form,
                msg='<strong>%s</strong>' %
                    (user.current.text('%d file(s) were sucessfully deleted!'%(deletedFilesNo)) ))

    # send deletion form
    wikiname = wikiutil.quoteWikiname(pagename)
    querytext = user.current.text('Please use the interactive user interface from UploadedFiles macro to delete files!')
    formhtml = """
<strong>%(querytext)s</strong>
""" % locals()

    return page.send_page(form, msg=formhtml)

def checkBackupDir():
    if not os.path.isdir(config.upload_backup_dir):
        os.mkdir(config.upload_backup_dir, 0777)
        os.chmod(config.upload_backup_dir, 0777)
    

def deleteFiles(theFiles):
    checkBackupDir()
    deletedFilesNo=0
    if type(theFiles) is not type([]):
        theFiles=(theFiles,)

    for each in theFiles:
        secureFilename=os.path.basename(each.value)
        secureFilepath=os.path.join(config.upload_dir,secureFilename)
        backupFilepath=os.path.join(config.upload_backup_dir,secureFilename)
        if os.path.isfile(secureFilepath):
            shutil.copyfile(secureFilepath,backupFilepath) 
            os.remove(secureFilepath)
        else:
            continue
        deletedFilesNo+=1
    return deletedFilesNo
