import cgi, string, os, sys
import posixpath, dospath, macpath
from MoinMoin import config, user, webapi, wikiutil
from MoinMoin.Page import Page

loadtextauto = 1                       # 1=read file at once
MAX_LENGTH=15*1024*1024

def printErrorMessage(actname,page,form):
    if actname in config.excluded_actions:
        page.send_page(form,
            msg='<strong>%s</strong>' %user.current.text('You are not allowed to upload files in this wiki!'))
        return 1
    elif config.edit_requires_login and (not user.current.valid):
        page.send_page(form,msg='<strong>%s</strong>' %
                user.current.text('<b>Please <a href="UserPreferences">log in</a> before uploading</b>'))
        return 1
    return 0

def execute(pagename, form):
    actname = string.split(__name__, '.')[-1]
    page = Page(pagename)
    error=printErrorMessage(actname,page,form)
    if error:
        return

    # check whether the user clicked the upload submit button
    if form.has_key('button') and form.has_key('ticket'):
        if not _checkTicket(form['ticket'].value):
            return page.send_page(form,msg='<strong>%s</strong>' %
                    user.current.text('Please use UploadFile action.'))
        checkUploadDir()
        msg=uploadFile(form)
        # Redirect to the original page
        return Page(pagename).send_page(form,msg='<strong>%s</strong>' %msg)

    # send upload form
    wikiname = wikiutil.quoteWikiname(pagename)
    ticket = _createTicket()
    button=user.current.text(' Upload ')
    formhtml="""
<form enctype="multipart/form-data" method="POST" action="%(wikiname)s">
<input type=file size=50 name=clientfile>
<input type="hidden" name="action" value="%(actname)s">
<input type="hidden" name="ticket" value="%(ticket)s">
<input type="submit" name="button" value="%(button)s">
</form>""" % locals()

    return page.send_page(form, msg=formhtml)


def _createTicket(tm = None):
    """Create a ticket using a site-specific secret (the config)"""
    import sha, time, types
    ticket = (tm or "%010x" % time.time())
    digest = sha.new()
    digest.update(ticket)

    cfgvars = vars(config)
    for var in cfgvars.values():
        if type(var) is types.StringType:
            digest.update(repr(var))
        
    return ticket + '.' + digest.hexdigest()


def _checkTicket(ticket):
    """Check validity of a previously created ticket"""
    timestamp = string.split(ticket, '.')[0]
    ourticket = _createTicket(timestamp)
    return ticket == ourticket    

def checkUploadDir():
    if not os.path.isdir(config.upload_dir):
        os.mkdir(config.upload_dir, 0777)
        os.chmod(config.upload_dir, 0777)

def uploadFile(form):
    if not form.has_key('clientfile'): 
        return "Error: no file was received"
    elif not form['clientfile'].filename:
        return "Error: file name is missing"
    filelength=int(os.environ["CONTENT_LENGTH"])
    if (filelength>MAX_LENGTH): 
        return """Error: file size is over the limit: %s byte"""%MAX_LENGTH

    fileinfo = form['clientfile']
    try: 
        srvrname = saveonserver(fileinfo)
    except:
        return '<h2>Error</h2><p>%s<p>%s' % (sys.exc_type, sys.exc_value)
    else:
        return user.current.text('File "%s" was sucessfully uploaded.') % (fileinfo.filename,)


def splitpath(origpath):                              # get file at end
    for pathmodule in [posixpath, dospath, macpath]:  # try all clients
        basename = pathmodule.split(origpath)[1]      # may be any server
        if basename != origpath:
            return basename                           # lets spaces pass
    return origpath                                   # failed or no dirs
    
def saveonserver(fileinfo):                           # use file input form data
    basename = splitpath(fileinfo.filename)           # name without dir path
    srvrname = os.path.join(config.upload_dir, basename)      # store in a dir if set
    if loadtextauto:
        filetext = fileinfo.file.read()                # reads text into string 
        srvrfile=open(srvrname, 'wb')
        srvrfile.write(filetext)           # save in server file
    else:
        srvrfile = open(srvrname, 'wb')                # else read line by line e.g. for huge files
        while 1:
            line = fileinfo.file.read(1024)
            if not line: break
            srvrfile.write(line)
    srvrfile.close()
    os.chmod(srvrname, 0666)   # make writeable: owned by 'nobody'
    return srvrname
