"""
    MoinMoin - AttentionPages macro

    Copyright (c) 2000 by Richard Jones <richard@bizarsoftware.com.au>
    Copyright (c) 2000 by Jurgen Hermann <jh@web.de>
    All rights reserved, see COPYING for details.

    This macro lists all pages which have an attention symbol in them along with 
    their last-modified date. Attention symbols are <!> and /!\.

    Usage:
    [[AttentionPages]]

    $Id$
"""

import string, re, time, os
from MoinMoin.Page import Page
from MoinMoin import user, config, wikiutil

def execute(macro, text, alert_re=re.compile(r'([^{]<!>)')):
    pages = wikiutil.getPageList(config.text_dir)
    l = []
    for page in pages:
        if len(page)>3 and string.lower(page[:4]) == 'help':
            continue
        page = Page(page)
        if alert_re.search(page.get_raw_body()):
            l.append((os.path.getmtime(page._text_filename()), page))
    l.sort()
    l.reverse()
    ret = ['<table class="wiki" border="1" cellspacing="0" cellpadding="3"><tr class="wiki"><th class="wiki">Page Name</th><th class="wiki">Last Modified</th></tr>']
    tnow = time.time()
    for mod, page in l:
        tdiff = int(tnow - float(mod)) / 60
        if tdiff < 1440:
            mod = "%dh&nbsp;%dm&nbsp;ago"%(tdiff/60, tdiff%60)
        elif tdiff < 43200:
            mod = "%dd&nbsp;%dh&nbsp;ago"%(tdiff/1440, (tdiff/60)%24)
        else:
            time_tuple = user.current.getTime(mod)
            mod = time.strftime("%D %M %Y", time_tuple)
        ret.append('<tr class="wiki"><td class="wiki"><a href="%s">%s</a></td><td class="wiki">%s</td></tr>'%(
            wikiutil.quoteFilename(page.page_name), page.split_title(), mod))
    ret.append("</table>")
    return string.join(ret, "\n")
