"""
    MoinMoin - FullSearch Macro

    Copyright (c) 2000-2001 by Jrgen Hermann <jh@web.de>
    All rights reserved, see COPYING for details.

    [[FullSearch]]
        displays a search dialog, as it always did

    [[FullSearch()]]
        does the same as clicking on the page title, only that
        the result is embedded into the page (note the "()" after
        the macro name, which is an empty argument list)

    [[FullSearch('HelpContents')]]
        embeds a search result into a page, as if you entered
        "HelpContents" into the search dialog

    $Id: FullSearch.py,v 1.2 2001/03/28 23:03:43 jhermann Exp $
"""

# Imports
import cgi, re, urllib, os
from MoinMoin import config, user, wikiutil

#_args_re_pattern = r'(((?P<hquote>[\'"])(?P<htext>.+?)(?P=hquote)(?:\s*,\s*)?)?(date)?)|'

_args_re_pattern = r'(((?P<hquote>[\'"])(?P<htext>.+?)(?P=hquote)(?:\s*,\s*)?)?(?P<sortkey>(date|match|dictionary)?))|'
def execute(macro, text, args_re=re.compile(_args_re_pattern)):
    # if no args given, invoke "classic" behavior
    if text is None:
        return macro._m_search("fullsearch", context=1)
    isEnglishWikiname=lambda _name:re.compile(r"(?:[%(u)s][%(l)s]+){2,}$"% {
                      'u':config.upperletters, 'l':config.lowerletters}).match(_name)
    # parse and check arguments
    args = args_re.match(text)
    if not args:
        return '<p><strong class="error">Invalid FullSearch arguments "%s"!</strong></p>' % (text,)

    needle = args.group('htext')
    literal = 1
    if not needle:
        # empty args means to duplicate the "title click" search (backlinks to page),
        # especially useful on "Category" type pages
        needle = macro.formatter.page.page_name
        literal = 1
        if not isEnglishWikiname(macro.formatter.page.page_name):
            needle=r'(\[|\[\")%s(\]|\"\])'%macro.formatter.page.page_name
            literal=0
    sortkey=args.group('sortkey')
    if not sortkey: sortkey='match'
    # do the search
    if needle=='all':
        pageList=wikiutil.getPageList(config.text_dir)
        pagecount=len(pageList)
        getPageFileName=lambda x:os.path.join(config.text_dir, wikiutil.quoteFilename(x))
        getModifiedTime=lambda x:os.path.getmtime(x)
        hits=[(getModifiedTime(getPageFileName(x)),x) for x in pageList]
        hits.sort(lambda x,y:cmp((-x[0],x[1]),(-y[0],y[1])))
        needle=''
    else:
        pagecount, hits = wikiutil.searchPages(needle, literal=literal,sortkey=sortkey)

    # generate the result
    result = macro.formatter.number_list(1)
    if sortkey=='date':
        _temp=resultWithDateSort(hits,needle,macro)
        return result+_temp+macro.formatter.number_list(0)

    for (count, pagename) in hits:
        result = result + macro.formatter.listitem(1)
        result = result + wikiutil.link_tag('%s?action=highlight&value=%s' %
            (wikiutil.quoteWikiname(pagename), urllib.quote(needle)),
            pagename)
        result = result + ' . . . . ' + `count` + [
            user.current.text(' match'),
            user.current.text(' matches')][count != 1]
        result = result + macro.formatter.listitem(0)
    result = result + macro.formatter.number_list(0)

    return result

def resultWithDateSort(hits,needle,macro):
    import time
    result=[]
    tnow = time.time()

    for (mod, pagename) in hits:
        tdiff = int(tnow - float(mod)) / 60
        if tdiff < 1440:
            mod = "%dh&nbsp;%dm&nbsp;ago"%(tdiff/60, tdiff%60)
        elif tdiff < 43200:
            mod = "%dd&nbsp;%dh&nbsp;ago"%(tdiff/1440, (tdiff/60)%24)
        else:
            time_tuple = user.current.getTime(mod)
            mod = time.strftime("%x", time_tuple)

        result.append(macro.formatter.listitem(1))
        if needle:
            _temp='%s?action=highlight&value=%s'
            _list=(wikiutil.quoteWikiname(pagename),needle)
        else:
            _temp='%s'
            _list=(wikiutil.quoteWikiname(pagename))
        result.append(wikiutil.link_tag(_temp %
            _list,
            pagename))
        result.append(' . . . . ' + mod)
        result.append(macro.formatter.listitem(0))
    return ''.join(result)
