"""
    MoinMoin - Action Handlers

    Actions are triggered by the user clicking on special links on the page
    (like the icons in the title, or the "EditText" link at the bottom). The
    name of the action is passed in the "action" CGI parameter.

    The sub-package "MoinMoin.action" contains external actions, you can
    place your own extensions there (similar to extension macros). User
    actions that start with a capital letter will be displayed in a list
    at the bottom of each page.

    User actions starting with a lowercase letter can be used to work
    together with a user macro; those actions a likely to work only if
    invoked BY that macro, and are thus hidden from the user interface.

    Copyright (c) 2000-2001 by Jrgen Hermann <jh@web.de>
    All rights reserved, see COPYING for details.

    $Id: wikiaction.py,v 1.17 2001/04/19 22:38:10 jhermann Exp $
"""

# Imports
import cgi, os, re, string, sys, time, urllib
from MoinMoin import config, user, util, wikiutil, webapi
from MoinMoin.Page import Page


#############################################################################
### Search
#############################################################################

def do_fullsearch(pagename, form):
    _open=_close=''
    # send http headers
    webapi.http_headers()
    if form.has_key('context') and (form['context'].value=="1"):
        _context=1
    else: _context=0

    if _context:  #backlinks
        _open='<SPAN class="closure">'
        _close='</SPAN>'

    # get parameters
    if form.has_key('value'):
        needle = form["value"].value
    else:
        needle = ''

    # send title
    wikiutil.send_title(user.current.text('Full text search for "%s"') % (needle,))

    # search the pages
    if _context:
        pagecount, hits, needle_re = wikiutil.searchPages(needle, literal=form.has_key('literal'),context=_context)
    else:
        pagecount, hits = wikiutil.searchPages(needle, literal=form.has_key('literal'),context=_context)
    #FIXED for context-result


    # print the result
    print "<UL>"
    for (count, page_name) in hits:
        print '<LI>%s'%_open + wikiutil.link_tag('%s?action=highlight&value=%s' %
            (wikiutil.quoteWikiname(page_name), urllib.quote_plus(needle)), page_name, tabindex=199)
        print ' . . . . ' + `count`
        print (user.current.text(' match'), user.current.text(' matches'))[count != 1]
        print _close
        if _context:
            wikiutil._printContexts(needle_re,page_name)
#        for each_match in surrounds:
#            each_match=cgi.escape(each_match)
#            each_match=string.replace(each_match,needle,"<strong>"+needle+"</strong>")
#            print '<dd><small>... '+each_match+' ...</small>'
    print "</UL>"

    print_search_stats(len(hits), pagecount)
    

def do_titlesearch(pagename, form):
    # TODO: check needle is legal -- but probably we can just accept any RE

    webapi.http_headers()

    if form.has_key('value'):
        needle = form["value"].value
    else:
        needle = ''

    wikiutil.send_title(user.current.text('Title search for "%s"') % (needle,))
    
    needle_re = re.compile(needle, re.IGNORECASE)
    all_pages = wikiutil.getPageList(config.text_dir)
    hits = filter(needle_re.search, all_pages)
    hits.sort()

    print "<UL>"
    for filename in hits:
        print '<LI>' + Page(filename).link_to(tabindex=199)
    print "</UL>"

    print_search_stats(len(hits), len(all_pages))


def print_search_stats(hits, pages):
    print "<p>" + user.current.text("%(hits)d hits out of %(pages)d pages searched.") % locals()


def do_highlight(pagename, form):
    if form.has_key('value'):
        needle = form["value"].value
    else:
        needle = ''

    try:
        needle_re = re.compile(needle, re.IGNORECASE)
    except re.error:
        needle = re.escape(needle)
        needle_re = re.compile(needle, re.IGNORECASE)

    Page(pagename).send_page(form, hilite_re=needle_re)


#############################################################################
### Misc Actions
#############################################################################

def do_diff(pagename, form):
    """ Handle "action=diff", checking for a "date=backupdate" parameter """
    webapi.http_headers()

    # send page title
    wikiutil.send_title(user.current.text('Diff for "%s"') % (pagename,), pagename=pagename)

    # try to get a specific date to compare to, or None for one-before-current
    try:
        diff_date = form['date'].value
        try:
            diff_date = float(diff_date)
        except StandardError:
            diff_date = None
    except KeyError:
        diff_date = None

    # get a list of old revisions, and back out if none are available
    oldversions = wikiutil.getBackupList(config.backup_dir, pagename)
    if not oldversions:
        print user.current.text("<b>No older revisions available!</b>")
        wikiutil.send_footer(pagename, showpage=1)
        return

    # get the filename of the version to compare to
    edit_count = 0
    if diff_date:
        for oldpage in oldversions:
            edit_count = edit_count + 1
            try:
                date = os.path.getmtime(os.path.join(config.backup_dir, oldpage))
            except EnvironmentError:
                continue
            if date <= diff_date: break
    else:
        # get the version before the current one
        edit_count = 1
        oldpage = oldversions[0]

    # build the diff command and execute it
    backup_file = os.path.join(config.backup_dir, oldpage)
    page_file = os.path.join(config.text_dir, wikiutil.quoteFilename(pagename))
    if not os.path.isfile(page_file): #FIXED for non-existent page_file(when it's deleted)
        page_file= os.path.join(config.data_dir, 'EmptyPage')
        _emptyPage=open(page_file,'w')
        _emptyPage.close()
    cmd = "diff -u %(backup)s %(page)s" % {
        "backup": backup_file,
        "page": page_file,
    }
    diff = util.popen(cmd, "r")
    lines = diff.readlines()
    rc = diff.close()
    ##print "cmd =", cmd, "<br>"
    ##print "rc =", rc, "<br>"

    # check for valid diff
    if not lines:
        print user.current.text("<b>No differences found!</b>")
        if edit_count:
            print '<p>' + user.current.text('The page was saved %(count)d%(times)s, though!') % {
                'count': edit_count,
                'times': (user.current.text(' time'), user.current.text(' times'))[edit_count != 1]}
        wikiutil.send_footer(pagename, showpage=1)
        return

    # Remove header lines
    if lines[0][0:3] == "---":
        del lines[0]
    if lines[0][0:3] == "+++":
        del lines[0]

    # Show date info
    print user.current.text('<b>Differences between version dated %s and %s') % (
        user.current.getFormattedDateTime(os.path.getmtime(backup_file)),
        user.current.getFormattedDateTime(os.path.getmtime(page_file)))

    if edit_count != 1:
        print user.current.text(' (spanning %d versions)') % (edit_count,)
    print '</b>'
    print '<div class="diffold">' + user.current.text('Deletions are marked like this.') + '</div>'
    print '<div class="diffnew">' + user.current.text('Additions are marked like this.') + '</div>'

    # Show diff
    for line in lines:
        marker = line[0]
        line = cgi.escape(line[1:])
        while line and line[-1] in ['\r', '\n']:
            line = line[:-1]
        stripped = string.lstrip(line)
        if len(line) - len(stripped):
            line = "&nbsp;" * (len(line) - len(stripped)) + stripped
        if not line:
            line = "&nbsp;"

        if marker == "@":
            print '<hr style="color:#FF3333">'
        elif marker == "\\":
            if stripped == "No newline at end of file\n": continue
            print '<div><font size="1" face="Verdana">%s&nbsp;</font></div>' % (line,)
        elif marker == "+":
            print '<div class="diffnew"><p>%s</div>' % (line,)
        elif marker == "-":
            print '<div class="diffold"><p>%s</div>' % (line,) #FIXED adding <p>
        else:
            print '<div>%s</div>' % (line,)

    wikiutil.send_footer(pagename, showpage=1)


def do_info(pagename, form):
    from stat import *

    webapi.http_headers()
    wikiutil.send_title(user.current.text('Info for "%s"') % (pagename,), pagename=pagename)
    currentpage = os.path.join(config.text_dir, wikiutil.quoteFilename(pagename))
    revisions = [currentpage]

    oldversions = wikiutil.getBackupList(config.backup_dir, pagename)
    if oldversions:
        for file in oldversions:
            revisions.append(os.path.join(config.backup_dir, file))

    print '<h2>' + user.current.text('Revision History') + '</h2>\n'
    print '<table border="1" cellpadding="2" cellspacing="0">\n'
    print '<tr><th>#</th>'
    print '<th>' + user.current.text('Date') + '</th>'
    print '<th>' + user.current.text('Size') + '</th>'
    if config.show_hosts:
        from MoinMoin import editlog
        log = editlog.EditLog()
        log.filter(pagename=pagename)
        print '<th>' + user.current.text('Editor') + '</th>'
    print '<th>' + user.current.text('Action') + '</th></tr>\n'
    count = 1
    for file in revisions:
        try: 
            st = os.stat(file) 
        except OSError:
            continue

        try:
            mtime = int(string.split(file, '.')[1])
        except IndexError:
            mtime = st[ST_MTIME]
        ##print count, mtime, st[ST_MTIME], "<br>"

        actions = ""
        if file != currentpage:
            actions = '%s&nbsp;<a href="%s/%s?action=recall&date=%s">%s</a>' % (
                actions,
                webapi.getScriptname(),
                wikiutil.quoteWikiname(pagename),
                os.path.basename(file)[len(wikiutil.quoteFilename(pagename))+1:],
                user.current.text('view'))
            actions = '%s&nbsp;<a href="%s/%s?action=diff&date=%d">%s</a>' % (
                actions,
                webapi.getScriptname(),
                wikiutil.quoteWikiname(pagename),
                mtime,
                user.current.text('diff'))
        print '<tr><td align="right">%d</td><td align="right">&nbsp;%s</td><td align="right">&nbsp;%d</td>' % (
            count,
            user.current.getFormattedDateTime(st[ST_MTIME]),
            st[ST_SIZE])

        if config.show_hosts:
            log.find(ed_time=mtime)
            print '<td>%s</td>' % (log.getEditor() or user.current.text("N/A"),)

        print '<td>&nbsp;%s</td></tr>\n' % (actions,)
        count = count + 1
        if count > 100: break
    print '</table>\n'
    
    wikiutil.send_footer(pagename, showpage=1)


def do_recall(pagename, form):
    Page(pagename, date=form['date'].value).send_page(form)


def do_show(pagename, form):
    Page(pagename).send_page(form)


def do_refresh(pagename, form):
    if form.has_key('arena') and form.has_key('key'):
        from MoinMoin import caching
        cache = caching.CacheEntry(form["arena"].value, form["key"].value)
        cache.remove()
        
    do_show(pagename, form)


def do_print(pagename, form):
    do_show(pagename, form)


def do_edit(pagename, form):
    Page(pagename).send_editor(form)


def do_savepage(pagename, form):
    pg = Page(pagename)
    try:
        savetext = form['savetext'].value
    except KeyError:
        savetext = ""
    try:
        datestamp = form['datestamp'].value
    except KeyError:
        datestamp = ""
    try:
        rstrip = form['rstrip'].value
    except KeyError:
        rstrip = 0
    savemsg = pg.save_text(savetext, datestamp, stripspaces=rstrip)
    pg.send_page(form, msg=savemsg)


def do_userform(pagename, form):
    savemsg=user.savedata(pagename, form)
    Page(pagename).send_page(form, msg=savemsg)


def do_bookmark(pagename, form):
    if form.has_key('time'):
        try:
            tm = int(form["time"].value)
        except StandardError:
            tm = time.time()
    else:
        tm = time.time()
    if user.current.name!=config.anonymousName:
        user.current.setBookmark(tm)
    Page(pagename).send_page(form)


#############################################################################
### Special Actions
#############################################################################

def do_raw(pagename, form):
    webapi.http_headers(["Content-type: text/plain"])

    try: 
        page = Page(pagename, date=form['date'].value)
    except KeyError:
        page = Page(pagename)

    sys.stdout.write(page.get_raw_body())
    sys.exit(0)


def do_format(pagename, form):
    # get the MIME type
    if form.has_key('mimetype'):
        mimetype = form['mimetype'].value
    else:
        mimetype = "text/plain"

    # try to load the formatter
    Formatter = util.importName("MoinMoin.formatter." +
        string.translate(mimetype, string.maketrans('/.', '__')), "Formatter") 
    if Formatter is None:
        # default to plain text formatter
        del Formatter
        mimetype = "text/plain"
        from formatter.text_plain import Formatter

    webapi.http_headers(["Content-Type: " + mimetype])
    print

    Page(pagename, formatter = Formatter()).send_page(form)
    sys.exit(0)

def do_rename(pagename, form): #ADDED
    webapi.http_headers()
    wikiutil.send_title(user.current.text('Rename for "%s"') % (pagename,),pagename=pagename)
    if config.edit_requires_login and (not user.current.valid):
        print '<strong>%s</strong>' %\
            user.current.text('<b>Please <a href="UserPreferences">log in</a> before renaming</b>') #%webapi.getScriptname()))
        wikiutil.send_footer('RenamePage', edittable=0,showpage=0)
        return
    print """
This action will change the name of a page in the Wiki. It will
also change any references to this page to refer to the new name.
If "All Versions?" is checked, it will be as if the page always had
the new name, and all references always used the new name. If not,
the page will effectively become a new Wiki page.
<p>
Note that the old page doesn't need to exist - nor does there have
to be any references to the page you're renaming. If you're unsure
about what the repurcussions of the rename will be, just hit the
'Show Rename Actions' button instead of 'Rename Page'.
<form method="get">
<input tabindex=100 type="hidden" name="action" value="saverename"> 
<table border=0>
<tr><td align=right>Old name:</td>
    <td><input tabindex=199 name="old" size=20 value="%s"></td></tr>
<tr><td align=right>New Name:</td>
    <td><input tabindex=199 name="new" size=20></td></tr>
<tr><td align=right>All versions?</td>
    <td><input tabindex=199 type="checkbox" name="all"></td></tr>
<tr><td align=right></td>
    <td><input tabindex=199 type="submit" name="submit" value="Show Rename Actions"></td></tr>
<tr><td align=right></td>
    <td><input tabindex=199 type="submit" name="submit" value="Rename Page"></td></tr>
</table>
</form>
"""%pagename
    wikiutil.send_footer('RenamePage', editable=0, showpage=0)

def do_saverename(pagename, form):
    webapi.http_headers()

    if config.edit_requires_login and (not user.current.valid):
        wikiutil.send_title('Renaming Error')
        print '<strong>%s</strong>' %\
            user.current.text('<b>Please <a href="UserPreferences">log in</a> before renaming</b>') #%webapi.getScriptname()))
        wikiutil.send_footer('RenamePage', edittable=0,showpage=0)
        return

    submit = form['submit'].value

    # make sure the arguments are valid
    if not form.has_key('new') or not form.has_key('old'):
        wikiutil.send_title('Renaming Error')
        print '<strong class="error">You must supply and old and a new name.</strong>'
        wikiutil.send_footer(pagename, editable=0, showpage=0)
        return

    # get the arguments
    old = form['old'].value
    new = form['new'].value
    if Page(new).exists():
        wikiutil.send_title('Renaming Error')
        print '<strong class="error">The new name already exists.</strong>'
        wikiutil.send_footer(pagename,editable=0,showpage=0)
        return
    all = 0
    if form.has_key('all'):
        all = 1

    # prepare for feedback
    wikiutil.send_title('Renaming "%s"'%old, pagename='RenamePage')
    if not all:
        print "<strong>Only modifying current pages</strong>"

    # find the files to rename (if the old exists)
    old_file = os.path.join(config.text_dir, wikiutil.quoteFilename(old))
    print "old file name : %s"%wikiutil.quoteFilename(old) #TEST
    rename = []
    try:
        os.stat(old_file)
    except OSError, value:
        if value.errno != 2:
            raise
    else:
        rename = [old_file]
    if all:
        rename = rename + map(lambda x: os.path.join(config.backup_dir, x),
            wikiutil.getBackupList(config.backup_dir, old))
    if rename:
        print "<dl><dt><strong>Files to be renamed:</strong>"
        for file_name in rename:
            print "<dd>",file_name
        print "</dl>"
############################
    isEnglishWikiname=lambda _name:re.compile(r"(?:[%(u)s][%(l)s]+){2,}$"% { #For not replacing every appearance of the word  
                          'u':config.upperletters, 'l':config.lowerletters}).match(_name)
    _sub=_filesub=r"\1%s\2"
    
    if isEnglishWikiname(old):
        search_re=re.compile(r'(|^|[^\w\[])(?:(?:\[|\[")%(_old)s(?:"\]|\])|(?:%(_old)s)(?=[^\w]|$))'% {\
                              '_old':wikiutil.quoteFilename(old)})
        _sub=r"\1%s"
        _filesub=r"\1%s"
        if not isEnglishWikiname(new): #from WikiName to Ű̸
            _sub=r"\1[%s]"
            _filesub=r"\1%s"
    else: #not a proper E wikiname. should be bracketed!!
        search_re = re.compile(r'(?:\[\"?)%s(?:\"?\])'%old)
        _sub=r"[%s]"
        _filesub=r"%s"
        if isEnglishWikiname(new): #from Ű̸ to WikiName
            _sub=r"%s"

############################
    # find the files that we need to change the contents of
#    isEnglishWikiname=lambda _name:re.compile(r"(?:[%(u)s][%(l)s]+){2,}$"% { #For not replacing every appearance of the word  
#                          'u':config.upperletters, 'l':config.lowerletters}).match(_name)
#    _sub=_filesub=r"\1%s\2"
#    
#    if isEnglishWikiname(old):
#        search_re = re.compile(r'(^|[^\w\[])(?<!")%s([^\w]|$)'%wikiutil.quoteFilename(old))
#        if not isEnglishWikiname(new): #from WikiName to Ű̸
#            _sub=r"\1[%s]\2"
#    else: #not a proper E wikiname. should be bracketed!!
#        search_re = re.compile(r'(\[\"?)%s(\"?\])'%old)
#        _filesub=r"%s"
#        if isEnglishWikiname(new): #from Ű̸ to WikiName
#            _sub=r"%s"

    change = []
    for page_name in wikiutil.getPageList(config.text_dir):
        page = Page(page_name)
        body = page.get_raw_body()
        if body is None:
            print '<strong class="error">Empty file %s?</strong>'%page_name
        if search_re.search(body):
            change.append((page, body))

    if all:
        for page_name in wikiutil.getBackupList(config.backup_dir):
            page_name, date = string.split(page_name, '.', 1)
            page = Page(page_name, date=date)
            body = page.get_raw_body()
            if body is None:
                print '<strong class="error">Empty file %s?</strong>'%page_name
            if search_re.search(body):
                change.append((page, body))
    def sortfun(a, b):
        a=a[0];b=b[0]
        if a.page_name == b.page_name:
            return cmp(a.prev_date, b.prev_date)
        return cmp(a.page_name, b.page_name)
    change.sort(sortfun)
    if change:
        print "<dl><dt><strong>Pages to be altered:</strong>"
        for (page, body) in change:
            if page.prev_date:
                print '<dd><a href="%s?action=recall&date=%s">%s '\
                    '(modified %s)</a>'%(page.page_name, page.prev_date,
                    page.page_name, time.strftime(config.datetime_fmt,
                    user.current.getTime(string.atof(page.prev_date))))
            else:
                print '<dd><a href="%s">%s</a>'%(page.page_name,
                    page.page_name)
        print "</dl>"

    # now see if we're actually supposed to do the change
    if submit == 'Rename Page':
        # do the contents first (just in case the page we're renaming
        # refers to itself
        for (page, body) in change:
            body = search_re.sub(_sub%new, body)
            # write directly to the file
            tmp = page._tmp_filename()
            open(tmp, 'wt').write(body)
            os.rename(tmp, page._text_filename())
        for file_name in rename:
            if not isEnglishWikiname(old):
                new_name=re.compile('%s'%wikiutil.quoteFilename(old)).sub(\
                                   _filesub%wikiutil.quoteFilename(new), file_name)
            else:                       
                new_name=search_re.sub(_filesub%wikiutil.quoteFilename(new), file_name)
            print "Renaming from %s to %s"%(file_name,new_name) #TEST
            os.rename(file_name, new_name)
            _executeSed="""sed -e 's/^%(from)s	/%(to)s	/g' <data/editlog >data/editlog.tmp"""%  \
                       {'from':os.path.basename(file_name).replace('/','\/'), \
                        'to':os.path.basename(new_name).replace('/','\/') }
            os.system(_executeSed)
            os.rename('data/editlog.tmp','data/editlog')


    wikiutil.send_footer('RenamePage', editable=0, showpage=0)



#############################################################################
### Dispatch Table
#############################################################################

_handlers = { 'fullsearch':  do_fullsearch,
              'titlesearch': do_titlesearch,
              'highlight':   do_highlight,
              'edit':        do_edit,
              'diff':        do_diff,
              'info':        do_info,
              'recall':      do_recall,
              'show':        do_show,
              'refresh':     do_refresh,
              'print':       do_print,
              'raw':         do_raw,
              'format':      do_format,
              'savepage':    do_savepage,
              'userform':    do_userform,
              'bookmark':    do_bookmark,
              'rename':      do_rename,
              'saverename':  do_saverename,
}

def getHandler(action):
    # check for excluded actions
    if action in config.excluded_actions or (config.access_requires_login and not  user.current.valid and action!='userform'):
        return None

    # check for and possibly return builtin action
    if _handlers.has_key(action):
        return _handlers[action]

    # try to load extension action
    from MoinMoin.action import extension_actions
    if action in extension_actions:
        # load extension action
        handler = util.importName("MoinMoin.action." + action, "execute")
        return handler

    # unknown action
    return None

